@@ -1,14 +1,23 @@
-# ========================================================================================================
+# ==================================================================================================
 # Return a custom distance function that calls registered functions of proxy
-# ========================================================================================================
+# ==================================================================================================
 
 ddist2 <- function(distance, control) {
-    symmetric <- isTRUE(control$symmetric)
     ## I need to re-register any custom distances in each parallel worker
     dist_entry <- proxy::pr_DB$get_entry(distance)
+    symmetric <- isTRUE(control$symmetric)
 
     ## Closures capture the values of the objects from the environment where they're created
     distfun <- function(x, centroids = NULL, ...) {
+        x <- any2list(x)
+        if (!is.null(centroids)) centroids <- any2list(centroids)
+        if (length(x) == 1L && is.null(centroids)) { # nocov start
+            return(structure(matrix(0, 1L, 1L),
+                             class = "crossdist",
+                             method = toupper(distance),
+                             dimnames = list(names(x), names(x))))
+        } # nocov end
+
         if (!is.null(control$distmat)) {
             if (inherits(control$distmat, "Distmat")) {
                 ## internal class, sparse or full
@@ -54,57 +63,91 @@ ddist2 <- function(distance, control) {
             dots <- dots[intersect(names(dots), valid_args)]
 
             ## variables/functions from the parent environments that should be exported
-            export <- c("distance", "dist_entry", "check_consistency", "enlist")
+            export <- c("distance", "dist_entry", "check_consistency", "enlist", "subset_dots")
 
-            if (is.null(centroids) && symmetric && !isTRUE(dots$pairwise)) {
-                if (dist_entry$loop && foreach::getDoParWorkers() > 1L && isTRUE(foreach::getDoParName() != "doSEQ")) {
-                    ## WHOLE SYMMETRIC DISTMAT WITH proxy LOOP IN PARALLEL
+            if (tolower(distance) %in% distances_included) {
+                ## DTWCLUST DISTANCES, LET THEM HANDLE OPTIMIZATIONS
+                d <- do.call(proxy::dist,
+                             enlist(x = x,
+                                    y = centroids,
+                                    method = distance,
+                                    dots = dots))
+
+            } else if (is.null(centroids) && symmetric && !isTRUE(dots$pairwise)) {
+                if (dist_entry$loop && foreach::getDoParWorkers() > 1L) {
+                    ## WHOLE SYMMETRIC DISTMAT IN PARALLEL
                     ## Only half of it is computed
-                    ## I think proxy can do this if y = NULL, but not in parallel
+                    ## proxy can do this if y = NULL, but not in parallel
+                    len <- length(x)
+                    seed <- get0(".Random.seed", .GlobalEnv, mode = "integer")
+                    d <- bigmemory::big.matrix(len, len, "double", 0)
+                    d_desc <- bigmemory::describe(d)
+                    assign(".Random.seed", seed, .GlobalEnv)
 
-                    ## strict pairwise as in proxy::dist doesn't make sense here,
-                    ## but it's pairwise between pairs
-                    dots$pairwise <- TRUE
-                    pairs <- call_pairs(length(x), lower = FALSE)
-                    pairs <- split_parallel(pairs, 1L)
+                    ids <- integer() ## 'initialize', so CHECK doesn't complain about globals
+                    foreach(
+                        ids = split_parallel_symmetric(len, foreach::getDoParWorkers()),
+                        .combine = c,
+                        .multicombine = TRUE,
+                        .noexport = c("d"),
+                        .packages = c(control$packages, "bigmemory"),
+                        .export = export
+                    ) %op% {
+                        if (!check_consistency(dist_entry$names[1L], "dist"))
+                            do.call(proxy::pr_DB$set_entry, dist_entry)
 
-                    d <- foreach(pairs = pairs,
-                                 .combine = c,
-                                 .multicombine = TRUE,
-                                 .packages = control$packages,
-                                 .export = export) %op% {
-                                     if (!check_consistency(dist_entry$names[1L], "dist"))
-                                         do.call(proxy::pr_DB$set_entry, dist_entry)
+                        dd <- bigmemory::attach.big.matrix(d_desc)
 
-                                     ## 'dots' has all extra arguments that are valid
-                                     dd <- do.call(proxy::dist,
-                                                   enlist(x = x[pairs[ , 1L]],
-                                                          y = x[pairs[ , 2L]],
-                                                          method = distance,
-                                                          dots = dots))
+                        if (isTRUE(attr(ids, "trimat"))) {
+                            ## assign upper part of lower triangular
+                            ul <- ids$ul
+                            if (length(ul) > 1L)
+                                dd[ul,ul] <- base::as.matrix(do.call(
+                                    proxy::dist,
+                                    enlist(x = x[ul],
+                                           y = NULL,
+                                           method = distance,
+                                           dots = dots)
+                                ))
+                            ## assign lower part of lower triangular
+                            ll <- ids$ll
+                            if (length(ll) > 1L)
+                                dd[ll,ll] <- base::as.matrix(do.call(
+                                    proxy::dist,
+                                    enlist(x = x[ll],
+                                           y = NULL,
+                                           method = distance,
+                                           dots = dots)
+                                ))
+                        } else {
+                            rows <- attr(ids, "rows")
+                            mat_chunk <- base::as.matrix(do.call(
+                                proxy::dist,
+                                enlist(x = x[rows],
+                                       y = x[ids],
+                                       method = distance,
+                                       dots = dots)
+                            ))
+                            ## assign matrix chunks
+                            dd[rows,ids] <- mat_chunk
+                            dd[ids,rows] <- t(mat_chunk)
+                        }
 
-                                     dd
-                                 }
+                        ## return from parallel foreach
+                        NULL
+                    }
 
-                    rm("pairs")
-                    D <- matrix(0, nrow = length(x), ncol = length(x))
-                    D[upper.tri(D)] <- d
-                    D <- t(D)
-                    D[upper.tri(D)] <- d
-                    d <- D
-                    rm("D")
+                    d <- d[,]
                     attr(d, "class") <- "crossdist"
                     attr(d, "dimnames") <- list(names(x), names(x))
 
                 } else {
                     ## WHOLE SYMMETRIC DISTMAT WITH CUSTOM LOOP OR SEQUENTIAL proxy LOOP
-                    ## maybe one of my distances, or one included in proxy by default, let it handle parallelization
                     d <- base::as.matrix(do.call(proxy::dist,
                                                  enlist(x = x,
                                                         y = NULL,
                                                         method = distance,
                                                         dots = dots)))
-
                     class(d) <- "crossdist"
                 }
 
