@@ -6,11 +6,20 @@
 #define CALLDEFpp(name, n) { "C_"#name, (DL_FUNC) &dtwclust::name, n }
 
 static R_CallMethodDef callMethods[] = {
+    CALLDEFpp(SparseDistmatIndices__new, 1),
+    CALLDEFpp(SparseDistmatIndices__getNewIndices, 4),
     CALLDEFpp(dba, 8),
-    CALLDEFpp(dtw_lb, 4),
+    CALLDEFpp(dtwb_loop, 10),
+    CALLDEFpp(dtw_lb, 5),
     CALLDEFpp(envelope, 2),
+    CALLDEFpp(force_lb_symmetry, 1),
+    CALLDEFpp(gak_loop, 9),
     CALLDEFpp(lbk, 4),
+    CALLDEFpp(lbk_loop, 9),
     CALLDEFpp(lbi, 6),
+    CALLDEFpp(lbi_loop, 11),
+    CALLDEFpp(sbd_loop, 10),
+    CALLDEFpp(tadpole, 8),
     CALLDEF(dtw_basic, 10),
     CALLDEF(logGAK, 8),
     CALLDEF(pairs, 2),
@@ -31,6 +40,7 @@ void register_functions() {
     DTWCLUST_REGISTER(logGAK);
     DTWCLUST_REGISTER(pairs);
     DTWCLUST_REGISTER(setnames_inplace);
+    DTWCLUST_REGISTER(tadpole);
     #undef DTWCLUST_REGISTER
 }
 
@@ -38,5 +48,4 @@ extern "C" void R_init_dtwclust(DllInfo* info) {
     register_functions();
     R_registerRoutines(info, NULL, callMethods, NULL, NULL);
     R_useDynamicSymbols(info, FALSE);
-    R_forceSymbols(info, TRUE);
 }
